@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2024 Jakub Kruszona-Zawadzki, Saglabs SA
+ * Copyright (C) 2025 Jakub Kruszona-Zawadzki, Saglabs SA
  * 
  * This file is part of MooseFS.
  * 
@@ -92,6 +92,7 @@
 #define CHLOG_KEEP_ATEND 300
 
 #define STORE_UNIT 60
+#define STORE_TIMEOUT (7200/(STORE_UNIT))
 
 static uint64_t metaversion;
 static uint64_t metaid;
@@ -103,7 +104,7 @@ static uint8_t verboselevel = 0;
 
 static uint32_t lastsuccessfulstore = 0;
 static double laststoretime = 0.0;
-static uint8_t laststorestatus = 0;
+static uint8_t laststorestatus = LASTSTORE_UNKNOWN;
 
 static uint64_t laststoremetaversion = 0;
 static uint32_t laststorechecksum = 0;
@@ -864,7 +865,7 @@ void meta_storeended(pid_t pid,int status) {
 			main_exit();
 		} else {
 			storestarttime = 0.0;
-			laststorestatus = (metasavermode)?3:0;
+			laststorestatus = (metasavermode)?LASTSTORE_CRC_STORED_BG:LASTSTORE_META_STORED_BG;
 			lastsuccessfulstore = main_time();
 			meta_process_crcdata();
 		}
@@ -1030,7 +1031,7 @@ int meta_storeall(int bg,uint8_t dontstore) {
 			lastsuccessfulstore = time(NULL);
 			laststoretime = monotonic_seconds()-storestarttime;
 			storestarttime = 0.0;
-			laststorestatus = 2; // Stored in foreground
+			laststorestatus = LASTSTORE_META_STORED_FG; // Stored in foreground
 			meta_process_crcdata();
 		}
 	} else {
@@ -1058,7 +1059,7 @@ void meta_download_status(uint8_t status) {
 	}
 	storestarttime = 0.0;
 	if (status==0) { // download successful
-		laststorestatus = 1; // Downloaded
+		laststorestatus = LASTSTORE_DOWNLOADED; // Downloaded
 		lastsuccessfulstore = main_time();
 		laststoremetaversion = 0;
 		laststorechecksum = 0;
@@ -1095,8 +1096,11 @@ void meta_store_task(void) {
 		offset = MetaSaveOffset;
 	}
 	htime = ( curtime / STORE_UNIT ) - offset;
+	if ((rhtime % 60)==0) {
+		changelog_rotate(ROTATE_FLAG_BROADCAST);
+	}
 	if ((htime % MetaSaveFreq) == 0) {
-		if (metasaverpid>=0 && last_store_htime + 120 * STORE_UNIT > rhtime) { // previus save still in progress - silently ignore this request
+		if (metasaverpid>=0 && last_store_htime + STORE_TIMEOUT > rhtime) { // previus save still in progress - silently ignore this request
 			return;
 		}
 		last_store_htime = rhtime;
@@ -1184,7 +1188,7 @@ int meta_mayexit(void) {
 }
 
 void meta_term(void) {
-	changelog_rotate(0);
+	changelog_rotate(ROTATE_FLAG_FOREGROUND);
 	for (;;) {
 		uint8_t status;
 		status = meta_storeall(0,0);
@@ -1325,6 +1329,7 @@ int meta_loadfile(const char *filename) {
 		fs_new();
 		chunk_newfs();
 		sessions_new();
+		sclass_new();
 		metaversion = 1;
 		metaid = 0;
 //		metaid = main_time();
@@ -1421,6 +1426,16 @@ int meta_loadall(void) {
 	int i;
 
 	if (emptystart) {
+		status = meta_check_metadatafile("metadata.mfs",&ver,&id);
+		if (status==META_CHECK_OK) {
+			mfs_log(MFSLOG_SYSLOG_STDERR,MFSLOG_INFO,"found valid file 'metadata.mfs' (version: %"PRIu64" ; id: %"PRIX64") in 'empty start' mode - will try to rename it",ver,id);
+			if (rename("metadata.mfs","metadata.mfs.back_emptystarted")<0) {
+				mfs_log(MFSLOG_ERRNO_SYSLOG_STDERR,MFSLOG_ERR,"can't rename metadata.mfs -> metadata.mfs.back_emptystarted");
+				return -1;
+			} else {
+				mfs_log(MFSLOG_ERRNO_SYSLOG_STDERR,MFSLOG_INFO,"file metadata.mfs has been renamed to metadata.mfs.back_emptystarted");
+			}
+		}
 		return 0;
 	}
 	if (allowautorestore) {
@@ -1957,6 +1972,40 @@ int meta_restore(void) {
 	return -1;
 }
 
+const char *meta_store_status_str(void) {
+	switch(laststorestatus) {
+		case LASTSTORE_DOWNLOADED:
+			return "DOWNLOADED";
+		case LASTSTORE_CRC_STORED_BG:
+			return "CRC ONLY IN BACKGROUND";
+		case LASTSTORE_META_STORED_BG:
+			return "STORED IN BACKGROUND";
+		case LASTSTORE_META_STORED_FG:
+			return "STORED IN FOREGROUND";
+	}
+	return "???";
+}
+
+void meta_log_extra_info(FILE *fd) {
+	fprintf(fd,"[meta]\n");
+	fprintf(fd,"meta id: 0x%016"PRIX64"\n",metaid);
+	fprintf(fd,"meta version: %"PRIu64"\n",metaversion);
+	if (laststoremetaversion>0) {
+		fprintf(fd,"last stored meta version: %"PRIu64"\n",laststoremetaversion);
+	}
+	if (lastsuccessfulstore>0) {
+		fprintf(fd,"metadata has been stored %d seconds ago\n",main_time()-lastsuccessfulstore);
+		fprintf(fd,"last metadata store mode: %s\n",meta_store_status_str());
+	}
+	if (metasaverpid>=0) {
+		fprintf(fd,"background sotre in progress; process pid: %d\n",(int)metasaverpid);
+	}
+	if (storestarttime>0) {
+		fprintf(fd,"store started %.2lf seconds ego\n",monotonic_seconds()-storestarttime);
+	}
+	fprintf(fd,"\n");
+}
+
 int meta_init(void) {
 	if (meta_prepare_data_structures()<0) {
 		return -1;
@@ -1973,6 +2022,7 @@ int meta_init(void) {
 	}
 	meta_reload();
 	main_reload_register(meta_reload);
+	main_info_register(meta_log_extra_info);
 	main_time_register(STORE_UNIT,0,meta_store_task);
 	main_mayexit_register(meta_mayexit);
 	main_destruct_register(meta_term);
