@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2024 Jakub Kruszona-Zawadzki, Saglabs SA
+Copyright (C) 2025 Jakub Kruszona-Zawadzki, Saglabs SA
 
 This file is part of MooseFS.
 
@@ -278,7 +278,7 @@ function applyAcidTab(force_init_acid_ready=false, force_init_acid_tab=false) {
 				if (table.acid_tab_sortedby!=null && table.acid_tab_indicator) {
 					table.acid_tab_indicator = document.createElement('span');
 					dir = (table.acid_tab_reversed)?'up':'down';
-					table.acid_tab_indicator.innerHTML = '<svg class="sort"><use xlink:href="#icon-sort-'+dir+'"/></svg>';				
+					table.acid_tab_indicator.innerHTML = '&#8205;<svg class="sort"><use xlink:href="#icon-sort-'+dir+'"/></svg>';				
 					table.acid_tab_sortedby.appendChild(table.acid_tab_indicator);
 				}
 				acid_tab.resort(table);
@@ -367,7 +367,7 @@ function applyAcidTab(force_init_acid_ready=false, force_init_acid_tab=false) {
 						if (table.acid_tab_indicator) {
 							table.acid_tab_indicator = document.createElement('span');
 							dir = (table.acid_tab_reversed)?'up':'down';
-							table.acid_tab_indicator.innerHTML = '<svg class="sort"><use xlink:href="#icon-sort-'+dir+'"/></svg>';
+							table.acid_tab_indicator.innerHTML = '&#8205;<svg class="sort"><use xlink:href="#icon-sort-'+dir+'"/></svg>';
 							hcell.appendChild(table.acid_tab_indicator);
 							
 						}
@@ -380,7 +380,7 @@ function applyAcidTab(force_init_acid_ready=false, force_init_acid_tab=false) {
 				i = 0;
 				while (smode==0 && i<tbody.rows.length) {
 					var cell = tbody.rows[i].cells[hcell.acid_tab_colid];
-					var text = acid_tab.getcelltext(cell,level);
+					var text = acid_tab.getcelltext(cell,level).split(' ')[0].replace(',','.').replace(/[\s']/g,'');
 					var rowspan = acid_tab.calcrowspan(tbody,i);
 					if (text!='') {
 						if (text.match(/^\s*(\+|-)?((\d+(\.\d+)?)|(\.\d+))(\s|%|$)/)) { // looks like a number
@@ -398,9 +398,15 @@ function applyAcidTab(force_init_acid_ready=false, force_init_acid_tab=false) {
 					var text = acid_tab.getcelltext(cell,level);
 					var rowspan = acid_tab.calcrowspan(tbody,i);
 					if (smode==1) {
-						text = parseFloat(text);
-						if (isNaN(text)) {
-							text = 0;
+						text = text.split(' ')[0].replace(',','.').replace(/[\s']/g,'');
+						j = parseFloat(text);
+						if (isNaN(j)) {
+							text = parseFloat(text.replace(/[^\d.-]/g, ''));
+							if (isNaN(text)) {
+								text = 0;
+							}
+						} else {
+							text = j;
 						}
 					}
 					k = newrows.length;
@@ -439,7 +445,7 @@ function applyAcidTab(force_init_acid_ready=false, force_init_acid_tab=false) {
 					if (table.acid_tab_indicator) {
 						table.acid_tab_indicator = document.createElement('span');
 						dir = (table.acid_tab_reversed)?'up':'down';
-						table.acid_tab_indicator.innerHTML = '<svg class="sort"><use xlink:href="#icon-sort-'+dir+'"/></svg>';
+						table.acid_tab_indicator.innerHTML = '&#8205;<svg class="sort"><use xlink:href="#icon-sort-'+dir+'"/></svg>';
 						hcell.appendChild(table.acid_tab_indicator);
 					}
 					acid_tab.remember(table);
